/* Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
<applet 
    	CODEBASE=".."
    	CODE="hamlet.AppButton.class" NAME=Root.Bisection width=130 height=25
    	align = bottom>
    	<param name=LABEL value="Newton's Method">

    If you see this, then Java is not running in your browser!<br>
    An applet would normally go here...<br>
</applet>
*/



/* Written by Elizabeth Odekirk. */
/* Program to give a lesson on the bisection method */

package Root;

import hamlet.*;
import java.awt.*;
import java.util.*;
import java.io.*;
import java.applet.Applet;


public class Root extends AppFrame {
	
  final public String version  = "v1.0a";  // Version number
    
  // Screen components
  Panel      inputs;
  StatusLabel  status;
  CoordPanel posGuessOut, negGuessOut, bisectOut, newtonOut;
  IntWLabel  numStepsOut;
  MenuBar    menubar;
  Menu       methodMenu;
  Menu       functionMenu;
  Menu       zoomOutMenu;
  Menu	     zoomMenu;
  MenuItem   zoomOut2Item, zoomOut5Item, zoomOut10Item;
  Menu       zoomInMenu;
  MenuItem   zoomIn2Item, zoomIn5Item, zoomIn10Item/*, zoomInRectItem*/;
  MenuItem   bisectItem, newtonItem;
  Menu       prefsMenu;
  MenuItem   showSpeedFrame;
  Menu	     windowMenu;
  MenuItem   relerrorItem;
  SpeedFrame speedFrame = new SpeedFrame(this);
  RelErrorWin relerror = new RelErrorWin();

  int curFunc = 0;          /* Start with the first function */
  FuncArray funcArray = new FuncArray ((Object) this);

  PlotPoints fnPoints[] = new PlotPoints[150];
  PlotPoints dropBall = new PlotPoints();
  boolean doPlotInCalc = true;
  /* Used with Newton's for the tangent line */
  PlotPoints lineBegin = new PlotPoints(),
             lineEnd   = new PlotPoints();
  PlotPoints fromXAxisBegin = new PlotPoints(), 
             fromXAxisEnd   = new PlotPoints();



  double posGuess = funcArray.array[0].posGuess,
         negGuess = funcArray.array[0].negGuess,
         bisect = 0,
         prevBisect = 0;

  PlotPoints posGuessPt,
             negGuessPt,
             bisectPt,
             guessPt;

  double guess = funcArray.array[0].guess;
  double prevGuess = 0;
  double nextGuess = 0;

  int numSteps = 1;        /* The number of bisection steps we've gone
			      through */
  
  int animDelay = 10;      /* Delay between steps in the animation */

  PlotInfo plotInfo = new PlotInfo();
  MyCanvas plot = new MyCanvas(plotInfo, this);
  /*  MySmallCanvas xplot = new MySmallCanvas (plotInfo, this);*/

  /* The methods for finding the root */
  public final static int BISECTION = 0;
  public final static int NEWTON    = 1;
  int rootMethod = Root.BISECTION;

  /* Which guess we are moving */
  public final static int POS_GUESS   = 0;
  public final static int NEG_GUESS   = 1;
  

  public Root (int method) 
  {
    rootMethod = method;

    int i;

    for (i = 0; i < 150; i++) {
      fnPoints[i] = new PlotPoints();
    }

    setUpScreen();
    helpText = 
      " This applet graphically shows two methods for finding a"	+"\n"+
      " root of a curve, the bisection method and Newtons method,"	+"\n"+
      " as they are described in Chapters 8 and 12, respectively,"	+"\n"+
      " of \"Introduction to Scientific Programming\" by Joseph L."     +"\n"+
      " Zachary.\n\n"							+
      //								//
      " File Menu\n"							+
      "   + Reset:  Returns to the default guess(es) for \n"		+
      "             this function\n"					+
      "   + Refresh: Refreshes the applet\n"				+
      "   + Hide:  Removes the applet from the display\n"		+
      "   + Quit:  Terminates the applet\n\n"				+
      //								//
      " Help Menu\n"							+
      "   + With Applet:  Displays this help dialog\n\n"		+
      //								//
      " Method Menu\n"							+
      "   + Bisection: Solve for a root using the bisection method\n"	+ 
      "   + Newton\'s Method: Solve for a root using\n"			+ 
      " 		      Newton\'s method\n\n"			+ 
      //								//
      " Function Menu\n"						+ 
      "   + Select a function to plot from the list\n\n"		+
      //								//
      " Zoom Menu\n"							+
      "   + Out\n"							+
      "       + By 2: \tDouble the amount of the plot seen\n"		+ 
      "       + By 5: \tMultiply by 5 the amount of the plot seen\n"	+ 
      "       + By 10: \tMultiply by 10 the amount of the plot seen\n" + 
      "   + In\n"							+
      "       + By 2: \tHalf the amount of the plot seen\n"		+ 
      "       + By 5: \tDivide by 5 the amount of the plot seen\n"	+ 
      "       + By 10: \tDivide by 10 the amount of the plot seen\n\n" + 
      //								//
      " Step Button\n"							+
      "   + One step (in the current method) towards finding the root\n\n"+
      //								//
      " Mouse Actions\n"						+ 
      "   + Mouse Click: Moves Guess\n"					+ 
      "       Bisection Method\n"					+
      "         + Clicking in an area where the function is positive\n" + 
      "           moves the positive guess\n"				+ 
      "         + Clicking in an area where the function is negative\n" + 
      "           moves the negative guess\n"				+ 
      "       Newtons Method\n"						+ 
      "         + Clicking moves the initial guess\n"			+
      "   + Mouse Drag: \tDefine a rectangle to zoom in on\n\n"		+ 
      //								//
      " Displayed Information\n"					+
      "   + Graph\n"							+
      "       + A graph of the chosen function\n"			+
      "       Bisection Method\n"					+
      "         + Two guesses, one positive (magenta) and one negative\n"+
      "           (red).  When you step, the bisection point shows up\n"+
      "           in blue between the two guesses and one of the\n"	+
      "           guesses moves to the bisection point.\n"		+
      "       Newton\'s Method\n"					+
      "         + One guess (magenta) and the tangent line (pink).\n"   +
      "           When you step, the ball drops along the tangent \n"   +
      "           line and then a line straight up from the x-axis to\n"+ 
      "           the function apears (pink) and the ball travels up \n"+
      "           it.\n"						+
      "   + Below the Graph\n"						+
      "       Bisection Method\n"					+
      "         + Positive Guess: The x and y positions\n"		+ 
      "             of the positive guess\n"		+
      "         + Bisection Point: The x and y positions of the point\n"+ 
      "             half-way between the positive and the\n" +
      "             negative guess.  When the y equals 0,\n" +
      "             we have found a root\n"		+
      "         + Negative Guess: The x and y positions\n"		+
      "             of the negative guess\n"		+
      "       Newton\'s Method\n"					+
      "         + Guess: The x and y positions of the current guess\n"    +
      "";


    pack();
    calcFunc();
  }


    //**************************************************************
    //  Start() runs each time the applet runs
    //**************************************************************/

  // Animates one step of the algorithm in a separate thread.  This
  // should not be invoked directly; it should be via appThread.start().

  synchronized public void nextStep() {

    double stepSize, slope;
    double plotStart, plotStop;
    double guessToMove = 0;
    double x, y;
    double prevX;
    PlotPoints prevDrop;
    int index;
    Graphics g = plot.getGraphics();
    int guessMoving;
    Color movingColor;


    // We're doing the bisection method

    if (rootMethod == BISECTION) {

      // Figure out if we're moving the positive point or the negative
      // point.
      if (funcArray.doFunc(bisect) < 0) {
	guessToMove = negGuess;
	negGuess = bisect;
	guessMoving = NEG_GUESS;
	movingColor = plotInfo.negGuessColor;
      }
      else {
	guessToMove = posGuess;
	posGuess = bisect;
	guessMoving = POS_GUESS;
	movingColor = plotInfo.guessesColor;
      }

      // Display the bisection point that lies halfway between
      // the two guesses.  Remember that we are currently showing it.
      plot.circle(plotInfo.bisect, g, plotInfo.bisectColor, true);
      plotInfo.showBisect = true;
      
      // Sleep for a while before moving the guess.
      try {
	Thread.sleep(100);
      } catch (InterruptedException e) {}

      // Slowly animate the motion of the guess.
      prevDrop = getPoint(guessToMove);
      stepSize = (bisect - guessToMove) / 50;
      for (x = guessToMove, index = 0; index <= 50; x += stepSize, index++) {
	dropBall.x = x;
	dropBall.y = funcArray.doFunc(x);
	synchronized (plotInfo) {
	  plot.circle(prevDrop, g, movingColor, true);
	  plot.circle(dropBall, g, movingColor, true);
	  prevDrop = new PlotPoints(dropBall);
	  if (guessMoving == POS_GUESS)
	    plotInfo.posGuess = new PlotPoints(dropBall);
	  else if (guessMoving == NEG_GUESS)
	    plotInfo.negGuess = new PlotPoints(dropBall);
	  getToolkit().sync();
	}

	// If the animation delay is zero, we briefly yield in case
	// we need to be interrupted.  Otherwise, we sleep.
	if (animDelay > 0)
	  try {
	    Thread.sleep(animDelay);
	  } catch (InterruptedException e) {}

      }

      // Remove the bisection point.
      plot.circle(plotInfo.bisect, g, plotInfo.bisectColor, true);
      plotInfo.showBisect = false;
      getToolkit().sync();

      // Update the statistics.
      prevBisect = bisect;
      bisect = bisection(posGuess,negGuess);
      relerror.text.setText("| oldBisection - newBisection |\n" + 
			    "| --------------------------- |\n" + 
			    "|         oldBisection        |\n\n");


      relerror.text.appendText("abs( ( " + prevBisect);

      if (bisect >= 0) {
	relerror.text.appendText(" - " + bisect);
      }
      else {
	relerror.text.appendText(" + " + -bisect);
      }

      relerror.text.appendText(" ) / " + prevBisect + " ) = " + 
			       Math.abs((prevBisect - bisect)/prevBisect));

      plotInfo.bisect.x = bisect;
      plotInfo.bisect.y = funcArray.doFunc(bisect);

      // Sleep for a while.
      try {
	Thread.sleep(300);
      } catch (InterruptedException e) {}

    } /* end rootMethod == BISECTION */

    /* The next guess is the place where the tangent
       line to the function at guess meets the x-axis */
    else if (rootMethod == NEWTON) {

      nextGuess = guess - 
	          (funcArray.doFunc(guess)/funcArray.doDiriv(guess));

      /* Remove the current guess */
      prevDrop = new PlotPoints(plotInfo.guess);
      prevDrop = new PlotPoints (plotInfo.guess);
      stepSize = (nextGuess - guess) / 10;
      slope = funcArray.doDiriv(guess);
      for (x = guess, index = 0; index <= 10; 
	                                x += stepSize, index++) {
	
	/* Synchronize on plot so that a repaint doesn't come
	   in the middle of the function. */
	synchronized (plot) {
	  dropBall.x = x;
	  dropBall.y = (slope * (x - guess ) ) 
	    + funcArray.doFunc(guess);
	  plot.circle(prevDrop, g, plotInfo.guessesColor, true);
	  plot.circle(dropBall, g, plotInfo.guessesColor, true);
	  getToolkit().sync();
	  prevDrop = new PlotPoints(dropBall);
	  plotInfo.guess = new PlotPoints(dropBall);
        }
	try {
	  Thread.sleep(animDelay);
	} catch (InterruptedException e) {}

      } 

      try {
	Thread.sleep(200);
      } catch (InterruptedException e) {}
      
      /* Now put in the line back up */
      /* This _should_ be done by getting the coordinates for just
       * above and below the screen, and going between them. */
      fromXAxisBegin.x = nextGuess;
      fromXAxisBegin.y = 10000;
      fromXAxisEnd.x   = nextGuess;
      fromXAxisEnd.y   = -10000;

      plotInfo.fromXAxis[0] = fromXAxisBegin;
      plotInfo.fromXAxis[1] = fromXAxisEnd;
      /* Need to change state so redraw will know to put it in */
      plotInfo.state = PlotInfo.DRAW_FROM_X_AXIS;
      /*System.out.println("Drawing line from " + fromXAxisBegin.y +
			 " to " + fromXAxisEnd.y +
			 " at " + nextGuess + "on x");*/
			 
      plot.line(plotInfo.fromXAxis, g, plotInfo.fromXAxisColor, true);
      

      /* And traverse the line */
      prevDrop = new PlotPoints(plotInfo.guess);
      stepSize =  funcArray.doFunc(nextGuess)/25;
      for (y = 0, index = 0; index <= 25; 
	                         y += stepSize, index++) {
	synchronized (plot) {
	  dropBall.x = nextGuess;
	  dropBall.y = y;
	
	  plot.circle(prevDrop, g, plotInfo.guessesColor, true);
	  plot.circle(dropBall, g, plotInfo.guessesColor, true);
	  getToolkit().sync();
	  prevDrop = new PlotPoints(dropBall);
	  plotInfo.guess = new PlotPoints(dropBall);
	}
	
	try {
	  Thread.sleep(animDelay);
	} catch (InterruptedException e) {}

      } 
      
      /* Now, delete the line from the x axis */
      plotInfo.state = PlotInfo.NO_CHANGE;
      plot.line(plotInfo.fromXAxis, g, plotInfo.fromXAxisColor, true);

      prevGuess = guess;
      guess = nextGuess;
      relerror.text.setText("| oldGuess - newGuess |\n" + 
			    "| --------------------|\n" + 
			    "|         oldBuess    |\n\n");


      relerror.text.appendText("abs( ( " + prevGuess);

      if (guess >= 0) {
	relerror.text.appendText(" - " + guess);
      }
      else {
	relerror.text.appendText(" + " + -guess);
      }

      relerror.text.appendText(" ) / " + prevGuess + " ) = " + 
			       Math.abs((prevGuess - guess)/prevGuess));

      plotInfo.guess = getPoint(guess);


      /* Delete the old tangent line */
      plot.line(plotInfo.tangLine, g, plotInfo.tangColor, true);
      
      plotStart = funcArray.array[curFunc].plotStart;
      plotStop = funcArray.array[curFunc].plotStop;

      /* Derive the new tangent line */
      lineBegin.x = plotInfo.min.x;
      lineBegin.y = funcArray.doDiriv(guess) *
	                        (plotInfo.min.x - guess) + 
				 funcArray.doFunc(guess);

      lineEnd.x = plotInfo.max.x;
      lineEnd.y = funcArray.doDiriv(guess) *
	                       (plotInfo.max.x - guess) + 
	                       funcArray.doFunc(guess);
      
      plotInfo.tangLine[0] = lineBegin;
      plotInfo.tangLine[1] = lineEnd;
      
      try {
	Thread.sleep(200);
      } catch (InterruptedException e) {}

	/*  System.out.println("Tang line: " + 
			     lineBegin.x + ", " +
			     lineBegin.y + " to " +
			     lineEnd.x + ", " +
			     lineEnd.y );*/
			     
      /* Draw in the new tangent line */
      plot.line(plotInfo.tangLine, g, plotInfo.tangColor, true);

    } /* end rootMethod == NEWTON */

    else {
      System.out.println("Unknown Root Finding Method");
    }

    setLabels();

    numStepsOut.setValue(++numSteps);

 } /* End run */

  PlotPoints getPoint(double x) {

    PlotPoints outPoint = new PlotPoints();
    outPoint.x = x;
    outPoint.y = funcArray.doFunc(x);
    return outPoint;
  }



  // Lays out all the graphical components.

  void setUpScreen() {

    // Title and top-level layout.
    setTitle("Finding Roots of Equations");
    setLayout(new BorderLayout(4, 4));  

    // Create a panel that displays the step button as well as the
    // number of steps made so far.
    Panel funcAndStep = new Panel();
    Button step = new Button("Step");
    numStepsOut = new IntWLabel ("Number Of Steps", numSteps);
    funcAndStep.setLayout(new GridLayout( 1, 2, 4, 4));
    funcAndStep.add(step);    
    funcAndStep.add(numStepsOut);
    
    // Create a label to display the method and function that are currently
    // being used.
    status = new StatusLabel (funcArray, rootMethod, curFunc);
    
    // Panels for display the coordinates of the positive, negative, and
    // bisection points.
    bisect = bisection(posGuess, negGuess);
    bisectOut = new CoordPanel("Bisection Point", bisect, 
			       funcArray.doFunc(bisect),
				 plotInfo.bisectColor);
    posGuessOut = new CoordPanel("Positive Guess", 
				 posGuess, 
				 funcArray.doFunc(posGuess),
				 plotInfo.guessesColor);
    negGuessOut = new CoordPanel("Negative Guess", negGuess, 
				 funcArray.doFunc(negGuess),
				 plotInfo.negGuessColor);


    // Panel for displaying all the coordinates
    Panel inputsBisection = new Panel();
    inputsBisection.setLayout(new GridLayout( 1, 3, 4, 4));
    inputsBisection.add(posGuessOut);
    inputsBisection.add(bisectOut);
    inputsBisection.add(negGuessOut);

    newtonOut = new CoordPanel("Guess", 
			       guess,
			       funcArray.doFunc(guess),
			       plotInfo.guessesColor);
    
    Panel inputsNewton = new Panel();
    inputsNewton.add(newtonOut);


    inputs = new Panel();
    inputs.setLayout(new CardLayout());
    inputs.add("Bisection", inputsBisection);
    inputs.add("Newtons Method", inputsNewton);

    Panel south = new Panel ();
    south.setLayout(new BorderLayout());
    south.add("North", funcAndStep);
    south.add("Center",inputs);

    plot.resize(200,200);
    plot.setForeground(Color.cyan);
    plot.setBackground(Color.gray);
    
    add("North", status);
    add("Center", plot);
    add("South", south);

    /* Menus */

    /* Method Menu */
    methodMenu   = new Menu("Method");
    methodMenu.add( bisectItem = new MenuItem("Bisection"));
    methodMenu.add( newtonItem = new MenuItem("Newtons Method"));


    /* Function Menu */
    functionMenu = new Menu("Function");
    for (int i = 0; i < funcArray.size; i++) {
      functionMenu.add(new MenuItem(funcArray.array[i].name));
    }
    
    
    /* Zoom Menu */
    zoomOutMenu = new Menu("Out");
    zoomOutMenu.add( zoomOut2Item = new MenuItem("By 2"));
    zoomOutMenu.add( zoomOut5Item = new MenuItem("By 5"));
    zoomOutMenu.add( zoomOut10Item = new MenuItem("By 10"));

    zoomInMenu = new Menu("In");
    zoomInMenu.add( zoomIn2Item = new MenuItem("By 2"));
    zoomInMenu.add( zoomIn5Item = new MenuItem("By 5"));
    zoomInMenu.add( zoomIn10Item = new MenuItem("By 10"));
/*    zoomInMenu.add( zoomInRectItem = new MenuItem("Zoom Rect"));*/

    zoomMenu = new Menu("Zoom");
    zoomMenu.add(zoomOutMenu);
    zoomMenu.add(zoomInMenu);

    /* Preferences Menu */
    prefsMenu = new Menu("Preferences");
    prefsMenu.add( showSpeedFrame = new MenuItem("Speed"));

    /* Windows Menu */
    windowMenu = new Menu("Window");
    relerrorItem = new MenuItem("Relative Error");
    windowMenu.add( relerrorItem );

    menubar = getMenuBar();
    menubar.add(methodMenu);
    menubar.add(functionMenu);
    menubar.add(zoomMenu);
    menubar.add(prefsMenu);
    menubar.add(windowMenu);

  }


/* This evaluates the function, and gets a list of points
 * for graphing.  It picks up all the defaults. */
public void calcFunc() {

    double xmin, xmax;
    double ymin, ymax;
    double stepSize;
    int index;
    int i;    double retval;
    double plotStart, plotStop;

    double x;

    PlotPoints min = new PlotPoints(), 
               max = new PlotPoints(); 

/*    showStatus("Calculating the plot");*/

    /* Only do this if this is the fist time we're
       showing the plot */
    plotStart = funcArray.array[curFunc].plotStart;
    plotStop = funcArray.array[curFunc].plotStop;
    posGuess = funcArray.array[curFunc].posGuess;
    negGuess = funcArray.array[curFunc].negGuess;
    guess    = funcArray.array[curFunc].guess;
    
    /* Would use xmin and xmax here (from plotInfo) */
    stepSize = (plotStop - plotStart )/ (double) 90;
    /*System.out.println("stepSize: " + stepSize);*/

    /* This is just to make the compiler happy */
    ymin = 0; ymax = 0;    xmin = 0; xmax = 0;

    for (x = plotStart, index = 0; x <= plotStop ; 
	                            x = x + stepSize, index++) {
      fnPoints[index].x = x;
      fnPoints[index].y = funcArray.doFunc(x);
      if (index == 0) {
	ymin = fnPoints[index].y;
	ymax = fnPoints[index].y;
	xmin = fnPoints[index].x;
	xmax = fnPoints[index].x;
      }
      if (fnPoints[index].y < ymin)
	ymin = fnPoints[index].y;
      if (fnPoints[index].y > ymax)
	ymax = fnPoints[index].y;
      if (fnPoints[index].x < xmin)
	xmin = fnPoints[index].x;
      if (fnPoints[index].x > xmax)
	xmax = fnPoints[index].x;
    }

    /* Y max and min can be off because we don't check all the values
     * (ie, sin and cos never get quite to 1). This makes our tick
     * marks more accurate for max and min. */ 
     ymax += .02 * (ymax - ymin); 
     ymin -= .02 * (ymax - ymin);

    min.x = xmin;
    min.y = ymin;
    max.x = xmax;
    max.y = ymax;


    /* We don't need to do this, except for the first time, but
       we do need to set fnPoints. */
    if (rootMethod == BISECTION) {
      posGuessPt = getPoint(posGuess);
      negGuessPt = getPoint(negGuess);
      bisect = bisection(posGuess, negGuess);
      bisectPt = getPoint(bisect);
      plotInfo.calcFnSet(fnPoints, index-1,
		     min, max, 
		     rootMethod, 
		     posGuessPt, negGuessPt, 
		     bisectPt);
    }
    else {
      guessPt = getPoint(guess);
      lineBegin.x = plotStart;
      lineBegin.y = funcArray.doDiriv(guess) *
	                        (plotStart - guess) + 
				 funcArray.doFunc(guess);

      lineEnd.x = plotStop;
      lineEnd.y = funcArray.doDiriv(guess) *
	                        (plotStop - guess) + 
				 funcArray.doFunc(guess);
      


      plotInfo.calcFnSet(fnPoints, index-1,
			  min, max,
			  lineBegin, lineEnd,
			  rootMethod,
			  guessPt);
    }


    setLabels();
    status.set(rootMethod, curFunc);
    plot.repaint();

  } /* End of calcFunc */


  void miniCalcFunc() {
    double xmin, xmax;
    double ymin, ymax;
    double stepSize;
    int index;
    int i;    double retval;
    double plotStart, plotStop;
    PlotPoints lineBegin = new PlotPoints(), 
               lineEnd   = new PlotPoints();

    double x;

    plotStart = plotInfo.min.x;
    plotStop = plotInfo.max.x;

    stepSize = (plotStop - plotStart )/ (double) 90;

    /* This is just to make the compiler happy */
    ymin = 0; ymax = 0;    xmin = 0; xmax = 0;

    for (x = plotStart, index = 0; x <= plotStop ; 
	                            x = x + stepSize, index++) {
      fnPoints[index].x = x;
      /* Here I should check, and if the number causes an overflow
	 (too large or small), then I put in maxDouble */
      fnPoints[index].y = funcArray.doFunc(x);
      if (index == 0) {
	ymin = fnPoints[index].y;
	ymax = fnPoints[index].y;
	xmin = fnPoints[index].x;
	xmax = fnPoints[index].x;
      }
      if (fnPoints[index].y < ymin)
	ymin = fnPoints[index].y;
      if (fnPoints[index].y > ymax)
	ymax = fnPoints[index].y;
      if (fnPoints[index].x < xmin)
	xmin = fnPoints[index].x;
      if (fnPoints[index].x > xmax)
	xmax = fnPoints[index].x;
    }

    if (rootMethod == NEWTON) {
      	  lineBegin.x = plotInfo.min.x;
	  lineBegin.y = funcArray.doDiriv(guess) *
	                        (plotInfo.min.x - guess) + 
				 funcArray.doFunc(guess);

	  lineEnd.x = plotInfo.max.x;
	  lineEnd.y = funcArray.doDiriv(guess) *
	                        (plotInfo.max.x - guess) + 
				 funcArray.doFunc(guess);
      
	  plotInfo.tangLine[0] = lineBegin;
	  plotInfo.tangLine[1] = lineEnd;
	/*  System.out.println("Tang line: " + 
			     lineBegin.x + ", " +
			     lineBegin.y + " to " +
			     lineEnd.x + ", " +
			     lineEnd.y );*/
			     
      }

    plotInfo.fnPoints = fnPoints;
    setLabels();
    plot.repaint();
  }


  /* Reads messages from AppletButton 
   *
   *   Bisection
   *   Newton
   *   Function <function #>
   *   Zoom In <amount>
   *   Zoom Out <amount>
   */
  protected void handleMessage (String msg) {
    StringTokenizer st = new StringTokenizer(msg);
    String curToken = "";
    boolean setFunc   = false;
    boolean setMethod = false;
    boolean setZoom   = false;
    boolean setPosGuess = false;
    boolean setNegGuess = false;
    double zoomAmount = 1;
    boolean gotAToken = false;

    try {
      while (st.hasMoreTokens()) {
	if (!gotAToken)
	  curToken = st.nextToken();
	else 
	  gotAToken = false;
	if (curToken.equalsIgnoreCase("Bisection")) {
	  rootMethod = Root.BISECTION;
	  setMethod = true;
	  if (st.hasMoreTokens()) {
	    curToken = st.nextToken();
	    if (curToken.equalsIgnoreCase("PosGuess") || 
		curToken.equalsIgnoreCase("NegGuess") ) {
	      int i;
	      for (i = 0; i < 2; i++) { 
		
		if (curToken.equalsIgnoreCase("PosGuess")) {
		  double posGuess;
		  /*posGuess = (Double.valueOf(st.nextToken())).
		                                        doubleValue();
		  PlotInfo.posGuess = getPoint(posGuess);*/
		  setPosGuess = true;
		}
		else if (curToken.equalsIgnoreCase("NegGuess")) {
		  double negGuess;
		  /*negGuess = (Double.valueOf(st.nextToken())).
		                                      doubleValue();
		  PlotInfo.negGuess = getPoint(negGuess);*/
		  setNegGuess = true;
		}
		curToken = st.nextToken();
	      }
	    }
	    else 
	      gotAToken = true;		// So we don't read another
	  }
	}/* End of "Bisection" */
	else if (curToken.equalsIgnoreCase("Newton")) {
	  rootMethod = Root.NEWTON;
	  setMethod = true;
	}
	else if (curToken.equalsIgnoreCase("Function")) {
	  int prevFunc = curFunc;
	  curFunc = (Integer.valueOf(st.nextToken())).intValue();
	  if (curFunc >= funcArray.size) {
	    curFunc = prevFunc;
	    System.err.println("Error in message - too many tokens");
	    System.err.println("Msg: " + msg);
	    return;
	  }
	  setFunc = true;
	} /* End "Function" */
	else if (curToken.equalsIgnoreCase("zoom")) {
	  curToken = st.nextToken();
	  if (curToken.equalsIgnoreCase("in")) {
	    zoomAmount = 
               1.0/(Double.valueOf(st.nextToken())).doubleValue();
	    setZoom = true;
	  }
	  else if (curToken.equalsIgnoreCase("out")) {
	    zoomAmount = 
	            (Double.valueOf(st.nextToken())).doubleValue();
	    setZoom = true;
	  }
	  else {
	    System.err.println("Error in message - Zoom has " +
			       "no option " + curToken);
	    System.err.println("Msg: " + msg);
	    return;
	  }
	} /* End "Zoom" */
	else {
	  System.err.println("Error in message - too many tokens");
	  System.err.println("Msg: " + msg);
	  return;
	} /* End of if else if....else */
      } /* End of while */
    } /* End of try */
    catch (NoSuchElementException e) {
      System.err.println("Error in message - format error");
      System.err.println("Msg: " + msg + "; Error token: " + 
			   e.getMessage());
      return;
    }
    catch (NumberFormatException e) {
      System.err.println("Error in message - format error in number");
      System.err.println("Msg: " + msg);
      System.err.println("Error token: " + e.getMessage());
      return;
    }
    if (setFunc || setMethod) {
      /*if (gotPosGuess) {
	double bisect = bisection(posGuess, negGuess);
	plotInfo.bisectPt = getPoint(bisect);
	miniCalcFunc();
      }*/
      calcFunc();
    }
    if (setZoom) {
      plot.zoom(zoomAmount);
      plot.repaint();
    }
  }/* End of handleMessage() */

    

  public boolean action (Event e, Object arg) {
    int newFn;

    /* If the parent handled it completely,
       don't do anything. */
    if (super.action(e, arg)) return(true);

    if ((e.target instanceof Button) 
	&& (e.id == Event.ACTION_EVENT)) {
      nextStep();
      return true;
    }

    else if ( e.target instanceof MenuItem ) {
      if ( e.target.equals( bisectItem ) || 
	   e.target.equals( newtonItem ) ) {
	if ( e.target.equals( bisectItem) ) 
	  rootMethod = BISECTION;
	else if (e.target.equals( newtonItem ) ) 
	  rootMethod = NEWTON;
	
	((CardLayout)inputs.getLayout()).show(inputs, 
					((MenuItem)e.target).getLabel()); 
	plotInfo.method = rootMethod;
	calcFunc();
	numSteps = 1;
	numStepsOut.setValue(numSteps);
	relerror.text.setText("");
	return true;
      }
      
      /* Preferences Menu */
      else if (e.target.equals( showSpeedFrame)) {
	if (e.target.equals( showSpeedFrame)) {
	  speedFrame.show();
	  speedFrame.toFront();
	}
	return true;
      }

      else if (e.target.equals( zoomOut2Item )) {
	plot.zoom(2.0);
	plot.repaint();
      }    
      else if (e.target.equals( zoomOut5Item )) {
	plot.zoom(5.0);
	plot.repaint();
      }    
      else if (e.target.equals( zoomOut10Item )) {
	plot.zoom(10.0);
	plot.repaint();
      }    
      else if (e.target.equals( zoomIn2Item )) {
	plot.zoom(0.5);
	plot.repaint();
      }    
      else if (e.target.equals( zoomIn5Item )) {
	plot.zoom(0.2);
	plot.repaint();
      }    
      else if (e.target.equals( zoomIn10Item )) {
	plot.zoom(0.1);
	plot.repaint();
      }    
      else if (e.target.equals( relerrorItem )) {
	relerror.show();
      }
      else {
	int i;
	 for (i = 0; i < funcArray.size; i++)  {
	   if (funcArray.array[i].name.equals(
		      ((MenuItem) e.target).getLabel()))
	     break;
	 }
	  curFunc = i;
	  calcFunc();
	  /* Reset the number of steps */
	  numSteps = 1;
	  numStepsOut.setValue(numSteps);
	return true;
      }

    }



    return false;
  }


  public void plotMouseDown(double x, double y) {
    double plotStart, plotStop;
    PlotPoints lineBegin = new PlotPoints(), 
               lineEnd   = new PlotPoints();

    	/*System.out.println( "Got mouseDown at " + x + 
			    ", " + y);*/
	if (rootMethod == BISECTION) {
	  if (funcArray.doFunc(x) > 0)
	    posGuess = x;
	  else
	    negGuess = x;

	  setLabels();
	  plotInfo.posGuess = getPoint(posGuess);
	  plotInfo.negGuess = getPoint(negGuess);
	  /* figure out and add bisection point here */
	  bisect = bisection(posGuess, negGuess);
	  plotInfo.bisect = getPoint(bisect);
	}
	else if (rootMethod == NEWTON) {
	  guess = x;
	  plotInfo.guess = getPoint(guess);

	  plotStart = funcArray.array[curFunc].plotStart;
	  plotStop = funcArray.array[curFunc].plotStop;
	  lineBegin.x = plotInfo.min.x;
	  lineBegin.y = funcArray.doDiriv(guess) *
	                        (plotInfo.min.x - guess) + 
				 funcArray.doFunc(guess);

	  lineEnd.x = plotInfo.max.x;
	  lineEnd.y = funcArray.doDiriv(guess) *
	                        (plotInfo.max.x - guess) + 
				 funcArray.doFunc(guess);
      
	  plotInfo.tangLine[0] = lineBegin;
	  plotInfo.tangLine[1] = lineEnd;
	  setLabels();
	 /* System.out.println("Tang line: " + 
			     lineBegin.x + ", " +
			     lineBegin.y + " to " +
			     lineEnd.x + ", " +
			     lineEnd.y );*/
			     
	}
      numSteps = 1;
      numStepsOut.setValue(numSteps);
      plot.repaint();
	

  }

  void setLabels() {
    if (rootMethod == BISECTION) {
      posGuessOut.setXvalue(posGuess);
      posGuessOut.setYvalue(funcArray.doFunc(posGuess));
      bisectOut.setXvalue(bisect);
      bisectOut.setYvalue(funcArray.doFunc(bisect));
      negGuessOut.setXvalue(negGuess);
      negGuessOut.setYvalue(funcArray.doFunc(negGuess));
    }
    else if (rootMethod == NEWTON) {
      newtonOut.setXvalue(guess);
      newtonOut.setYvalue(funcArray.doFunc(guess));
    }
  }

    

  double bisection(double x1, double x2) {
    return (x1 - (x1-x2)/2);
  }



} /* end of FP class */


class MyCanvas extends Canvas
{

  int inset = 20;
  double ymax, ymin, xmax, xmin;
  int radius = 5;               // Size of the circles
  /* These are used in the axis */
  double xaxisPosy, yaxisPosx;
  int tickHalfLen = 2;
  PlotCoord pc;
  /* Used by newtons */
  

  PlotInfo pi;
  Root     parentRoot;


  /* for rubberbanding when selecting zoom area */
  double tempMouseDownx, tempMouseDowny;
  int xfrom, yfrom;
  int oldTox, oldToy;
  int xprev, yprev; 
  boolean prevSet;

  

  MyCanvas(PlotInfo piIn, Root parentRootIn) {
    super();
    xmin = -100;
    xmax = 100;
    ymin = -2;
    ymax = 2;
    pi = piIn;
    parentRoot = parentRootIn;
  }


  public Dimension preferredSize () {
    FontMetrics fm = getFontMetrics(getFont());
    return(new Dimension(200, 200));
  }

  public void setXmin(double newXmin) {
    xmin = new Double(newXmin).floatValue();
  }

  public void setYmin(double newYmin) {
    ymin = new Double(newYmin).floatValue();
  }

  public double getXmin() {
    return(xmin);
  }

  public void setXmax(double newXmax) {
    xmax = new Double(newXmax).floatValue();
  }

  public void setYmax(double newYmax) {
    ymax = new Double(newYmax).floatValue();
  }

  public void zoom(double zoomAmount) {
    double midx, midy, xdist, ydist;
    double tempminx, tempmaxx;

    xdist = Math.abs (pi.max.x - pi.min.x);
    ydist = Math.abs (pi.max.y - pi.min.y);
    midx = pi.min.x + .5*xdist;
    midy = pi.min.y + .5*ydist;

    tempminx = midx - zoomAmount*(xdist/2);
    tempmaxx = midx + zoomAmount*(xdist/2);

    if ( (tempmaxx - tempminx) < Math.pow(10.0, -13)) {
      System.err.println("Cannot zoom smaller");
      repaint();
      return;
    }

    pi.min.x = midx - zoomAmount*(xdist/2);
    pi.max.x = midx + zoomAmount*(xdist/2);
    pi.min.y = midy - zoomAmount*(ydist/2);
    pi.max.y = midy + zoomAmount*(ydist/2);

    /*System.out.println("In Zoom: xdist: " + xdist +
		               " ydist: " + ydist + 
		               " midx: "  + midx  +
		               " midy: "  + midy  +
	               " min: "   + pi.min.x + ", " + pi.min.y +
	               " max: "   + pi.max.x + ", " + pi.max.y ); */
		             
		               
    parentRoot.miniCalcFunc();

  }

  /* This is for when you want to zero in on part of the graph */
  synchronized public boolean mouseDown(Event e, int xpos, int ypos) {
    double x, y;

    /* If we are zooming in on a rectangle in the graph, 
     * get the coords */
      yfrom = ypos;
      xfrom = xpos;

      tempMouseDownx = pc.coordToX(xpos);
      tempMouseDowny = pc.coordToY(ypos);
      return true;
  }
    

  /* Only do something if we are dragging a rectangle for
   * zooming.  */
  synchronized public boolean mouseDrag(Event e, int xpos, int ypos) {
    Graphics g = getGraphics();
    int xto, yto;

    xto = xpos;
    yto = ypos;

    /*    System.out.println( "Mousedrag at coords:" + 
		       xto + ", " + yto ); */

    g.setXORMode(Color.pink);
    if (prevSet) {
      drawBox(xfrom, xprev, yfrom, yprev, g);
    }
    drawBox(xfrom, xto, yfrom, yto, g);
    getToolkit().sync();
    xprev = xto;
    yprev = yto;
    prevSet = true;
    g.setPaintMode();
    return true;
  }


  /* Used to draw a box when I'm rubberbanding the rect region. */
  synchronized void drawBox(int xfrom, int xto, int yfrom, int yto, 
                            Graphics g) {
    g.drawLine(xfrom, yfrom, xfrom, yto);
    g.drawLine(xfrom, yto, xto, yto);
    g.drawLine(xto, yto, xto, yfrom);
    g.drawLine(xto, yfrom, xfrom, yfrom);
  }


  /* Only need to do something here if we are defining a zoom rectangle */
  synchronized public boolean mouseUp(Event e, int xpos, int ypos) {
    Graphics g = getGraphics();
    double tempMouseUpx, tempMouseUpy;
    double tempminx, tempmaxx;
    double diff;

    prevSet = false;
    tempMouseUpx = pc.coordToX(xpos);
    tempMouseUpy = pc.coordToY(ypos);
    
    /*System.out.println( "MouseDown up coords:" + 
		       xpos + ", " + ypos + 
		       " which on the screen are " + 
		       tempMouseUpx + ", " + 
		       tempMouseUpy );*/
    
    /* Here, check to see also if they went just a small ways */
    /* If they just click in one spot, don't change anything */
    if ((xpos == xfrom) ||  
	(ypos == yfrom) ||
        ( Math.abs(xpos - xfrom) < 5) || 
        ( Math.abs(ypos - yfrom) < 5) ) {
      double xdown, ydown;
      xdown = pc.coordToX(xfrom);
      ydown = pc.coordToY(yfrom);
      parentRoot.plotMouseDown(xdown, ydown);
      /*System.out.println("Did a Guess");*/
      return true;
    }

    
    diff = Math.abs(tempMouseUpx - tempMouseDownx);

    if ( diff < Math.pow(10.0, -13)) {
      repaint();
      System.err.println("Cannot zoom smaller");
      return true;
    }


    if (tempMouseDownx < tempMouseUpx) {
      pi.min.x = tempMouseDownx;
      pi.max.x = tempMouseUpx;
    }
    else {
      pi.min.x = tempMouseUpx;
      pi.max.x = tempMouseDownx;
    }

    if (tempMouseDowny < tempMouseUpy) {
      pi.min.y = tempMouseDowny;
      pi.max.y = tempMouseUpy;
    }
    else {
      pi.min.y = tempMouseUpy;
      pi.max.y = tempMouseDowny;
    }
      
    parentRoot.miniCalcFunc();
    return true;
  }

  /* When we are repainting the screen */
  synchronized public void update (Graphics g) {
    int x, y;
    Dimension d = size();
    Rectangle r = this.bounds();
    double xcur = -1, ycur = -1;
    int xcoord = -1, ycoord = -1;
    int xcoordBegin, ycoordEnd,  ycoordBegin, xcoordEnd;
    int xcoordPrev, ycoordPrev;
    Color prevColor;
    int i, j;

    xmax = pi.max.x;
    xmin = pi.min.x;
    ymax = pi.max.y;
    ymin = pi.min.y;
    

      synchronized (pi) {
      g.setColor(this.getBackground());
      g.fillRect(0, 0, r.width, r.height);
      g.setColor(this.getForeground());
    

      /* Draw a rectangle around the Canvas */
      g.drawRect(0,0, d.width - 1, d.height - 1);

      /* Set up the transformation between cartesian coordinates
	 and screen coordinates. */
      pc = new PlotCoord (pi.max.x, pi.max.y, 
			  pi.min.x, pi.min.y, 
			  d.width, d.height, inset);

      /* If there is only one value for y, set up a little bit of room
	 in the graph */
      if (pi.max.y == pi.min.y) {
	pi.max.y = pi.max.y + 2;
	pi.min.y = pi.min.y - 2;
      }

      drawAxis(g);

      /*System.out.println("Xmin: " + xmin + "  Xmax: " + xmax);
      System.out.println("Ymin: " + ymin + "  Ymax: " + ymax);*/

      xcoordPrev = -1;
      ycoordPrev = -1;

      /* if both prev-coord and cur-coord are off the screen,
	 don't draw the line */
      double yTop, yBottom, xprev = -1, yprev = -1;
      yTop    = pc.coordToY(0);
      yBottom = pc.coordToY(d.height);
      
      /*System.out.println("");*/
      for (i = 0; i < pi.numFnPoints; i++) {
	  if (i != 0) {
	    xcoordPrev = xcoord;
	    ycoordPrev = ycoord;
	    xprev = xcur;
	    yprev = ycur;
	  }

	  xcur = pi.fnPoints[i].x;
	  ycur = pi.fnPoints[i].y;

	  xcoord = pc.xcoord(xcur);
	  ycoord = pc.ycoord(ycur);

	  /* if both prev-coord and cur-coord are off the screen,
	   * don't draw the line */
	  if (i != 0 ) {
	    if (yprev < yBottom && ycur < yBottom)
	      continue;
	    if (yprev > yTop && ycur > yTop)
	      continue;
	  }
	    

	  /*System.out.println("xcur: " + xcur + 
			     " ycur: " + ycur +
			     " xcoord: " + xcoord +
			     " ycoord: " + ycoord);*/

	  if (i != 0) 
	    g.drawLine(xcoordPrev, ycoordPrev, xcoord, ycoord);

      }

      if (pi.method == Root.BISECTION) {
	circle(pi.posGuess, g, pi.guessesColor, true);
	circle(pi.negGuess, g, pi.negGuessColor, true);
	if (pi.showBisect == true)
	  circle(pi.bisect,   g, pi.bisectColor, true);
      }
      else if (pi.method == Root.NEWTON) {
	circle(pi.guess, g, pi.guessesColor, true);
	line(pi.tangLine, g, pi.tangColor, true);
	if (pi.state == PlotInfo.DRAW_FROM_X_AXIS)
	  line(pi.fromXAxis, g, pi.fromXAxisColor, true);
      }

    pi.state = PlotInfo.NO_CHANGE;
    }
    


  } /* end update */


  /*  PAINT
   */ 
  /** Draws a box around this panel, plus other stuff. */
  public void paint(Graphics g) {
    update(g);
  } /* end paint */


  void line(PlotPoints pts[], Graphics g, Color color, boolean xor) {
    Color prevColor, backgrColor;
    int xcoord1, ycoord1, xcoord2, ycoord2;

    xcoord1 = pc.xcoord(pts[0].x);
    ycoord1 = pc.ycoord(pts[0].y);
    xcoord2 = pc.xcoord(pts[1].x);
    ycoord2 = pc.ycoord(pts[1].y);

    /*System.out.println("For line, coordinates are " +
		       xcoord1 +  ", " + ycoord1 + "  " +
		       xcoord2 +  ", " + ycoord2);*/
		       
    
    backgrColor = getBackground();
    prevColor = g.getColor();
    if (xor) {
      g.setXORMode(color);
      g.setColor(backgrColor);
    }
    else {
      g.setColor(color);
    }
    g.drawLine(xcoord1, ycoord1, xcoord2, ycoord2);
    g.setColor(prevColor);
    g.setPaintMode();
  }    
    

  /* This function is synchronized because I do not want paint
     and this function to be going on at the same time. */
  synchronized void circle
    (PlotPoints pt, Graphics g, Color color, boolean xor) {

    Color prevColor, backgrColor;
    int xcoord, ycoord;

    xcoord = pc.xcoord(pt.x);
    ycoord = pc.ycoord(pt.y);

    
    backgrColor = getBackground();
    prevColor = g.getColor();
    if (xor) {
      g.setXORMode(color);
      g.setColor(backgrColor);
    }
    else {
      g.setColor(color);
    }
    g.fillOval(xcoord-radius, ycoord-radius, 2*radius, 2*radius);
    g.setColor(prevColor);
    g.setPaintMode();
    /* System.out.println("circle: " + pt.x + ", " + pt.y);*/
  }    


  /* DRAW AXIS
   */
  void drawAxis(Graphics g)
  {

    Color prevColor;

    double xaxisStart = pi.max.x, xaxisEnd = pi.min.x;
    double yaxisStart = pi.max.y, yaxisEnd = pi.min.y;

    /* Axis rules:
       If 0 is on the map, print out the axis there.
       If 0 is not on the map, print out the axis on the 
       minimum. */

    if (xmin < 0  && xmax > 0) 
      yaxisPosx = 0;
    else
      yaxisPosx = xmin;

    if (ymin < 0  && ymax > 0) 
      xaxisPosy = 0;
    else
      xaxisPosy = ymin ;


    
    prevColor = g.getColor();
    g.setColor(Color.black);
    /* Draw x axis. */
    g.drawLine(pc.xcoord(xaxisStart), pc.ycoord(xaxisPosy), 
	       pc.xcoord(xaxisEnd), pc.ycoord(xaxisPosy));
    /* Draw y axis. */
    g.drawLine(pc.xcoord(yaxisPosx), pc.ycoord(yaxisStart),
	       pc.xcoord(yaxisPosx), pc.ycoord(yaxisEnd));

    Font f = new Font ("Dialog", Font.PLAIN, 10);
    g.setFont(f);

    drawAllTicks(g);
    g.setColor(prevColor);

  }


  void drawAllTicks(Graphics g) {
    double mult, multPart, dif, curTick, logNum = 0;
    int i = 0;
    double foo = 0;
    double lwr, lwrMiniticks;   /* The lowest tick on the graph */

    /* draw a tick at 0 if it is on the graph */
    if (xmin < 0  && xmax > 0) {
      drawXTick(0, g);
    }

    if (ymin < 0  && ymax > 0) {
      drawYTick(0, g);
    }

    /* draw x ticks */
    dif = xmax - xmin;
    
    try {
      logNum = Math.log(dif)/Math.log(10);
    } catch (ArithmeticException e) {
      System.out.println("Got an arithmetic exception in ticks");  
    }

    logNum = Math.floor(logNum);

    /*System.out.println("xmax is " + xmax + ", xmin is " + xmin + 
                        ", i is " + i + ", logNum is " + logNum + 
		       ", dif is " + dif);*/

    if (dif/Math.pow(10,logNum) < 3) {
      foo = 2;
    }

/*    System.out.println("xmax is " + xmax + ", xmin is " + xmin + 
                        ", i is " + i + ", logNum is " + logNum + 
		       ", dif is " + dif);*/

    if (foo == 0)
      mult = Math.pow(10, logNum);
    else
      mult = 2 * Math.pow(10, logNum-1);

    lwr = xmin / mult;
    lwr = Math.ceil(lwr);
    lwr = lwr * mult;

    for ( curTick = lwr; curTick <= xmax; curTick += mult) {
       drawXTick(curTick, g);
     }

    /* draw y ticks */
    dif = ymax - ymin;
    
    /* log10(dif) */
    try {
      logNum = Math.log(dif)/Math.log(10);
    } catch (ArithmeticException e) {
      System.out.println("Got an arithmetic exception in ticks");  
    }

    logNum = Math.floor(logNum);

    /*System.out.println("ymax is " + ymax + ", ymin is " + ymin + 
               ", i is " + i + ", logNum is " + logNum + ", dif is " + dif);*/

    mult = Math.pow(10, logNum);
    multPart = mult/5;

    lwrMiniticks = ymin / multPart;
    lwrMiniticks = Math.ceil(lwrMiniticks);
    lwrMiniticks = lwrMiniticks * multPart;

    lwr = ymin / mult;
    lwr = Math.ceil(lwr);
    lwr = lwr * mult;

    curTick = 0;

    /* Draw the very first small ticks */
    for (curTick = lwrMiniticks, i = 0; curTick < lwr; curTick += multPart) 
      drawYMiniTick(curTick + i*multPart, g);

    for ( curTick = lwr; curTick <= ymax; 
	                        curTick += mult) {
       drawYTick(curTick, g);
       for (i = 1; i < 5 && ((curTick + i*multPart) <= ymax); i ++) {
	 drawYMiniTick(curTick + i*multPart, g);
       }

     }
  }

  void drawYTick(double y, Graphics g) {
    int xcoord, ycoord;
    int offset;

    ycoord = pc.ycoord(y);
    xcoord = pc.xcoord(yaxisPosx);
    
    if (y == 0)
      offset = 10;
    else
      offset = 5;

    g.drawLine(xcoord + tickHalfLen, 
	       ycoord , xcoord - tickHalfLen, ycoord);
    g.drawString(new Float(y).toString(),
		 xcoord + 5, ycoord + offset);
  }

  void drawYMiniTick(double y, Graphics g) {
    int xcoord, ycoord;

    ycoord = pc.ycoord(y);
    xcoord = pc.xcoord(yaxisPosx);
    
    g.drawLine(xcoord + tickHalfLen/2, 
	       ycoord , xcoord - tickHalfLen/2, ycoord);
  }

  void drawXTick(double x, Graphics g) {
    int xcoord, ycoord;

    xcoord = pc.xcoord(x);
    ycoord = pc.ycoord(xaxisPosy);

    g.drawLine(xcoord, ycoord + tickHalfLen, 
	       xcoord , ycoord - tickHalfLen);
    g.drawString(new Float(x).toString(),
		 xcoord - 5, ycoord + 15);

  }

} /* End MyPlot */



/* CLASS        PLOT COORD
 *
 * 
 */
class PlotCoord {
  
  double xstretch = 1;
  double ystretch = 1;
  double xshift = 0;
  double yshift = 0;
  double ywholeScreen;       
  int inset;             /* How far from the edge the display is */

  PlotCoord(double xmax, double ymax, double xmin, double ymin, 
	    int width, int height, int insetin ) {

    inset = insetin;

    xstretch = (width - (2 * inset))/ (xmax - xmin);
    ystretch = (height - (2 * inset)) / (ymax - ymin);

    xshift = -xmin;
    yshift = -ymin;

    ywholeScreen = ymax - ymin;

  }

  /* Return the X and Y drawing coordinates on the canvas. */
  int xcoord(double x) {
    double xcoordFloat;
    
    xcoordFloat = (x + xshift) * xstretch + inset;
    if (xcoordFloat >= (float)Integer.MAX_VALUE)
      xcoordFloat = (float)Integer.MAX_VALUE;
    if (xcoordFloat <= (float)Integer.MIN_VALUE)
      xcoordFloat = (float)Integer.MIN_VALUE;
      
    return(new Long(Math.round(xcoordFloat)).intValue());
  }

  int ycoord(double y) {
    double ycoordFloat;
    
    ycoordFloat = (ywholeScreen - (y + yshift)) * ystretch + inset;
    /*System.out.println("Min Integer: " + Integer.MIN_VALUE + 
		       " Max Integer: " + Integer.MAX_VALUE);
    System.out.println("Before change: " + y +
		       " After change: " + ycoordFloat);*/

    if (ycoordFloat >= (float)Integer.MAX_VALUE)
      ycoordFloat = (float)Integer.MAX_VALUE;
    if (ycoordFloat <= (float)Integer.MIN_VALUE)
      ycoordFloat = (float)Integer.MIN_VALUE;
    /* This will not show up anyway, and if the value gets too
       large it causes problems. */
    /* This will cause problems, like if I'm drawing a line and only
       part of it is on the screen.  It would be better to somehow
       scale it to the max negative the screen can allow. */
    /*if (ycoordFloat < 0)
      ycoordFloat = -1.0;*/
      
    return(new Long(Math.round(ycoordFloat)).intValue());
  }
       

  /* Return the X and Y real coords on canvas. */
  double coordToX(int xcoord) {
    double x;

    x = (xcoord - inset) / xstretch - xshift;
    return(x);
  }

  double coordToY(int ycoord) {
    double y;

    y = -( ((ycoord - inset) / ystretch) - ywholeScreen + yshift);
    return(y);
  }
       

}

class PlotInfo {

  PlotPoints fnPoints[];                /** For both methods **/
  int        numFnPoints;
  PlotPoints min,
             max;
  int        method;
  int        state;         /* what are we painting now? */

  PlotPoints posGuess,                  /** For Bisection **/
             negGuess,
             bisect;
  boolean    showBisect = false;

  PlotPoints guess;                   /** For Newtons method **/
  PlotPoints tangLine[]  = new PlotPoints[2];            
  PlotPoints fromXAxis[] = new PlotPoints[2];

  Color guessesColor,
	negGuessColor,
        bisectColor,
        tangColor,
        fromXAxisColor;
  

  /* Different states */
  public final static int NO_CHANGE            = 0;
  public final static int DRAW_FROM_X_AXIS     = 15;
  

  PlotInfo() {
    guessesColor  = Color.magenta;
    negGuessColor = Color.red;
    bisectColor   = Color.blue;
    tangColor     = Color.orange;
    fromXAxisColor = Color.pink;
  }
  

  /* Setting up for bisection */
  synchronized public void  
             calcFnSet(
                        PlotPoints plotPointsIn[],
		        int        numFnPointsIn,
		        PlotPoints minIn, 
		        PlotPoints maxIn, 	   
		        int        methodIn,
		        PlotPoints posGuessIn,
		        PlotPoints negGuessIn,
		        PlotPoints bisectIn) 
 {
   fnPoints     = plotPointsIn;
   numFnPoints  = numFnPointsIn;
   min = minIn;
   max = maxIn;
   method   = methodIn;
   posGuess = posGuessIn;
   negGuess = negGuessIn;
   bisect   = bisectIn; 
   
   /* In here, just set all that stuff */
 }

  /* Setting up for Newton */
  synchronized public void  
             calcFnSet(
		       PlotPoints plotPointsIn[], 
		       int        numFnPointsIn,
		       PlotPoints minIn, 
		       PlotPoints maxIn, 	   
		       PlotPoints lineBeginIn,
		       PlotPoints lineEndIn, 
		       int        methodIn,
		       PlotPoints guessIn)
 {
   fnPoints = plotPointsIn; 
   numFnPoints  = numFnPointsIn;
   min = minIn; 
   max = maxIn; 	   
   tangLine[0] = lineBeginIn;
   tangLine[1]   = lineEndIn; 
   method    = methodIn;
   guess     = guessIn;
   /* In here, just set all that stuff */

 }


}

class SpeedFrame extends Frame {

  Scrollbar scroll = new Scrollbar( Scrollbar.HORIZONTAL, 
				   10		/*initial value*/ ,
				   100,		/*visible*/
				   0,           /*minimum*/
				   50		/*maximum*/
				   );
  Root      topFrame;
  Label     nameLabel   = new Label("Speed of Animation");
  Button    closeButton = new Button("Close");

  public Dimension preferredSize () {
    return (new Dimension(300, 100));
  }


  public SpeedFrame (Root topFrameIn) {
    topFrame = topFrameIn;
    Panel closePanel = new Panel();
    closePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    closePanel.add(closeButton);
    /*add("North",  nameLabel);*/
    add("Center", scroll);
    add("North",  closePanel);
    setTitle("Setting the Speed");
    pack();
  }
    
  public boolean handleEvent(Event e) {
    if (e.target.equals(scroll) && 
	( (e.id == Event.SCROLL_ABSOLUTE) || 
	  (e.id == Event.SCROLL_LINE_DOWN) || 
	  (e.id == Event.SCROLL_LINE_UP)) )   {
	  topFrame.animDelay = scroll.getValue();
      return true;
    }
    else if (e.target.equals(closeButton) && 
	     e.id == Event.ACTION_EVENT) {
      hide();
      return true;
    }
    return false;
  }

}
